#include "Debug.h"
#include "DEV_Config.h"
#include "AT24C02.h"

#define Init_data (0xff)
#define Ploy (0x31)

uint8_t crc8(uint8_t init, uint8_t poly, uint8_t *data, uint16_t len);
uint16_t reboot_cnt;
void setup() {
  uint8_t cnt;

  uint8_t read_data[10];
  uint8_t write_data[10];
  DEV_ModuleInit();

  AT24C02_Read(0x00, read_data, 3);

  for (cnt = 0; (read_data[2] != crc8(Init_data, Ploy, read_data, 2)) && (cnt < 3); cnt++) {
    write_data[0] = 0;
    write_data[1] = 0;
    write_data[2] = crc8(Init_data, Ploy, write_data, 2);
    AT24C02_Write(0x00, write_data, 3);
    DEV_Delay_ms(10);
    AT24C02_Read(0x00, read_data, 3);
  }
  if (cnt >= 3) {
    Serial.printf("EEPROM Test error");
    reboot_cnt = 10000;
    return;
  }
  reboot_cnt = read_data[0] << 8;
  reboot_cnt += read_data[1];

  if (reboot_cnt >= 1000) {
    reboot_cnt = 0;
  }
  Serial.printf("Reboot cnt = %d\r\n", reboot_cnt);
  reboot_cnt++;

  write_data[1] = reboot_cnt & 0xff;
  write_data[0] = (reboot_cnt >> 8) & 0xff;
  write_data[2] = crc8(Init_data, Ploy, write_data, 2);
  AT24C02_Write(0x00, write_data, 3);
}

void loop() {
  if (reboot_cnt == 10000) { 
    return;
  }
  Serial.printf("Reboot cnt = %d\r\n", reboot_cnt);
  DEV_Delay_ms(1000);
}

/******************************************************************************
function: crc8
info:   Read the temperature and humidity sensor values and displayed on the screen
parameter:
    init    :   crc initial value
    poly    :   crc polynomial
    data    :   Data to be validated
    len     :   Data length
******************************************************************************/
uint8_t crc8(uint8_t init, uint8_t poly, uint8_t *data, uint16_t len) {
  uint8_t crc = init;
  while (len--) {
    crc ^= *data++;
    for (uint8_t i = 0; i < 8; i++) {
      if (crc & 0x80) {
        crc = (crc << 1) ^ poly;
      } else {
        crc = crc << 1;
      }
    }
  }
  return crc;
}