#include <stdint.h>
/**
 * Copyright (c) 2021 mjcross
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#include "nec_receive.h"

#if defined(ARDUINO_ARCH_RP2040)
// SDK types and declarations
#include "Arduino.h"
#include "pico/stdlib.h"
#include "hardware/pio.h"
#include "hardware/clocks.h" // for clock_get_hz()
// import the assembled PIO state machine program
#include "nec_receive.pio.h"

int sm = 0;
int offset = 0;
PIO nec_rec_pio = pio0;

#endif

union ir_data
{
    uint32_t raw;
    struct
    {
        uint8_t address;
        uint8_t inverted_address;
        uint8_t data;
        uint8_t inverted_data;
    };
} f;

// Claim an unused state machine on the specified PIO and configure it
// to receive NEC IR frames on the given GPIO pin.
//
// Returns: the state machine number on success, otherwise -1
bool nec_rx_init(uint pin_num)
{
#if defined(ARDUINO_ARCH_RP2040)

    // Find a free SM on one of the PIO's
    sm = pio_claim_unused_sm(nec_rec_pio, false); // don't panic
    // Try pio1 if SM not found
    if (sm < 0)
    {
        nec_rec_pio = pio1;
        sm = pio_claim_unused_sm(nec_rec_pio, true); // panic if no SM is free
        if (sm < 0)
        {
            return false;
        }
    }

    // install the program in the PIO shared instruction space
    uint offset;
    if (pio_can_add_program(nec_rec_pio, &nec_receive_program))
    {
        offset = pio_add_program(nec_rec_pio, &nec_receive_program);
    }
    else
    {
        return false; // the program could not be added
    }

    // configure and enable the state machine
    nec_receive_program_init(nec_rec_pio, sm, offset, pin_num);
    return true;
#endif
}
// Validate a 32-bit frame and store the address and data at the locations
// provided.
//
// Returns: `true` if the frame was valid, otherwise `false`
bool nec_decode_frame(uint32_t frame, uint8_t *p_address, uint8_t *p_data)
{

    // access the frame data as four 8-bit fields
    //
    f.raw = frame;

    // a valid (non-extended) 'NEC' frame should contain 8 bit
    // address, inverted address, data and inverted data
    if (f.address != (f.inverted_address ^ 0xff) || f.data != (f.inverted_data ^ 0xff))
    {
        return false;
    }

    // store the validated address and data
    *p_address = f.address;
    *p_data = f.data;
    return true;
}
bool nec_receive_available(void)
{
#if defined(ARDUINO_ARCH_RP2040)
    return (!pio_sm_is_rx_fifo_empty(nec_rec_pio, sm));
#endif
}

uint32_t nec_receive_raw(void)
{
#if defined(ARDUINO_ARCH_RP2040)
    return pio_sm_get(nec_rec_pio, sm);
#endif
    // ;
}

void nec_receive_data(uint8_t *p_address, uint8_t *p_data)
{
    uint32_t raw_data = nec_receive_raw();
    nec_decode_frame(raw_data, p_address, p_data);
}
