#include "WS2812B.h"

#if defined(ARDUINO_ARCH_RP2040)
int sm = 0;
int offset = 0;
PIO ws2812_pio = pio0;
#endif

bool WS2812_Init(uint8_t ws2812_pin, bool rgbw) {

#if defined(ARDUINO_ARCH_RP2040)

  // Find a free SM on one of the PIO's
  sm = pio_claim_unused_sm(ws2812_pio, false);  // don't panic
  // Try pio1 if SM not found
  if (sm < 0) {
    ws2812_pio = pio1;
    sm = pio_claim_unused_sm(ws2812_pio, true);  // panic if no SM is free
    if (sm < 0) {
      return false;
    }
  }
  offset = pio_add_program(ws2812_pio, &ws2812_program);

  ws2812_program_init(ws2812_pio, sm, offset, ws2812_pin, 800000, rgbw);
  return true;
#elif defined(ESP32)
  return true;
#else
  return false;
#endif
}

void WS2812_Write_Byte(uint32_t rgb_data) {
#if defined(ARDUINO_ARCH_RP2040)
  pio_sm_put_blocking(ws2812_pio, sm, rgb_data );
#elif defined(ESP32)

#endif
}
void WS2812_Write_nByte(uint32_t *rgb_data, uint16_t len) {
  for (; len > 0; len--) {
    WS2812_Write_Byte(*rgb_data++);
  }
}
void WS2812_Clear(uint32_t rgb_data, uint16_t len) {
  for (; len > 0; len--) {
    WS2812_Write_Byte(rgb_data);
  }
}
uint32_t RGB_Data_Convert(uint8_t red,uint8_t green,uint8_t blue)
{
  return (uint32_t)(red<<24)+(green<<16)+(blue<<8);
}
