/*****************************************************************************
* | File      	:   D0-KEY.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include <stdio.h>
#include "pico/stdlib.h"

#define Key_Pin (3)
#define LED_Pin (10)
/******************************************************************************
function: key_init
info:   Set GPIO to input and pull up
parameter:
    gpio    :   Set gpio to key
******************************************************************************/
void key_init(uint gpio)
{
    gpio_init(gpio);
    gpio_set_dir(gpio, GPIO_IN);
    gpio_pull_up(gpio);
}
/******************************************************************************
function: led_init
info:   Set GPIO to output
parameter:
    gpio    :   Set gpio to led
******************************************************************************/
void led_init(uint gpio)
{
    gpio_init(gpio);
    gpio_set_dir(gpio, GPIO_OUT);
}
/******************************************************************************
function: main
info:   When the key is pressed, led on. 
        When the key is released, led off and output the press time by serial port.
parameter:
******************************************************************************/
int main()
{
    uint32_t i;
    stdio_init_all();
    // init key and led
    key_init(Key_Pin);
    led_init(LED_Pin);
    // led off
    gpio_put(LED_Pin, true);

    printf("Key Test\r\n");
    printf("Wait for the key to be pressed\r\n");

    while (true)
    {
        // Detect when the key is pressed
        if (gpio_get(Key_Pin) == false)
        {
            sleep_ms(2);
            if (gpio_get(Key_Pin) == false)
            {
                // led on
                gpio_put(LED_Pin, false);
                printf("The key is pressed \r\n");
                for (i = 0; !gpio_get(Key_Pin); i++)
                {
                    sleep_ms(1);
                }
                if (i > 0)
                {
                    printf("The key was held down %dms \r\n", i);
                }
                // led off
                gpio_put(LED_Pin, true);
            }
        }
    }

    return 0;
}
