/*****************************************************************************
* | File      	:   D1-LED.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/pwm.h"

#define LED_Pin (10)

/******************************************************************************
function: PWM_init
info:   Set GPIO to pwm
parameter:
    gpio    :   Set gpio to pwm
    div     :   Frequency division
******************************************************************************/
void PWM_init(uint gpio,float div)
{
    uint slice_num;
    gpio_set_function(gpio, GPIO_FUNC_PWM);
    slice_num = pwm_gpio_to_slice_num(gpio);
    pwm_set_wrap(slice_num, 10000);
    pwm_set_chan_level(slice_num, gpio%2, 0);
    pwm_set_clkdiv(slice_num,div);
    pwm_set_enabled(slice_num, true);
}
/******************************************************************************
function: PWM_set_freq
info:   Set PWM frequency
parameter:
    gpio    :   Select the GPIO you want to set
    freq    :   Set frequency
******************************************************************************/
void PWM_set_freq(uint gpio,uint16_t freq)
{
    uint slice_num;
    float div =12500.0/freq;
    if(div>=256)
    {
        printf("freq too low,maximum is 256+(15/16)\r\n");
        div=255;
    }
    //printf("freq= %d, div = %f \r\n",freq,div);
    slice_num = pwm_gpio_to_slice_num(gpio);
    
    pwm_set_clkdiv(slice_num,div);
}
/******************************************************************************
function: PWM_set_duty
info:   Set PWM duty
parameter:
    gpio    :   Select the GPIO you want to set
    duty    :   Set duty
******************************************************************************/
void PWM_set_duty(uint gpio,uint8_t duty)
{
    uint slice_num;
    if(duty>100)
    {
        printf("duty too big,maximum is 100\r\n");
        duty=100;
    }
    slice_num = pwm_gpio_to_slice_num(gpio);
    pwm_set_chan_level(slice_num, gpio%2, duty*100);

}
/******************************************************************************
function: main
info:   Modify duty to achieve the effect of breathing lamp
parameter:
******************************************************************************/
int main()
{
    //set sys clock 125Mhz
    set_sys_clock_khz(125*1000,true);
    stdio_init_all();
    //set pwm freq = 125*1000*1000/50/1000=2500HZ
    PWM_init(LED_Pin,50);
    
    printf("Hello, PWM!");
    
    // set pwm duty 30%
    PWM_set_duty(LED_Pin,30);

    while (true)
    {
        for(uint16_t i=0;i<100;i++)
        {
           PWM_set_duty(LED_Pin,i);
           sleep_ms(10); 
        }
        for(uint16_t j=100;j>0;j--)
        {
           PWM_set_duty(LED_Pin,j);
           sleep_ms(10);

        }
    }
    return 0;
}
