/*****************************************************************************
* | File      	:   I2C0-6-DOF.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include <stdio.h>
#include "pico/stdlib.h"

#include "OLED_1in5.h"
#include "GUI_Paint.h"
#include "DEV_Config.h"
#include "QMI8658.h"

/******************************************************************************
function: main
info:   The screen display the title, acceleration and gyroscope readings
parameter:
******************************************************************************/
int main()
{
    float acc[3], gyro[3];
    unsigned int tim_count = 0;
    uint8_t *BlackImage;
    uint16_t Imagesize = ((OLED_1in5_WIDTH % 2 == 0) ? (OLED_1in5_WIDTH / 2) : (OLED_1in5_WIDTH / 2 + 1)) * OLED_1in5_HEIGHT;

    if (DEV_Module_Init() != 0)
    {
        return -1;
    }
    //Initialize the OLED
    OLED_1in5_Init();
    OLED_1in5_Clear(0);

    if ((BlackImage = (uint8_t *)malloc(Imagesize)) == NULL)
    {
        while (1)
        {
            printf("Failed to apply for black memory...\r\n");
        }
    }
    printf("Paint_NewImage\r\n");
    Paint_NewImage(BlackImage, OLED_1in5_WIDTH, OLED_1in5_HEIGHT, 0, WHITE);
    Paint_SetScale(16);

    printf("Drawing\r\n");
    // 1.Select Image
    Paint_SelectImage(BlackImage);
    Paint_Clear(BLACK);
    //Initialize the 6DOF sensor
    QMI8658_init();
    while (true)
    {
        char str[20];
        QMI8658_read_xyz(acc, gyro, &tim_count);
        printf("acc_x   = %4.3fmg , acc_y  = %4.3fmg , acc_z  = %4.3fmg\r\n", acc[0], acc[1], acc[2]);
        printf("gyro_x  = %4.3fdps, gyro_y = %4.3fdps, gyro_z = %4.3fdps\r\n", gyro[0], gyro[1], gyro[2]);
        printf("tim_count = %d\r\n", tim_count);
        Paint_DrawString_EN(5, 10, "6DOF TEST", &Font12, WHITE, BLACK);
        Paint_DrawString_EN(5, 25, "IC:QMI8658", &Font12, WHITE, BLACK);

        sprintf(str, "acc_x :%.2f", acc[0]);
        Paint_DrawString_EN(5, 40, str, &Font12, WHITE, BLACK);
        sprintf(str, "acc_y :%.2f", acc[1]);
        Paint_DrawString_EN(5, 55, str, &Font12, WHITE, BLACK);
        sprintf(str, "acc_z :%.2f", acc[2]);
        Paint_DrawString_EN(5, 70, str, &Font12, WHITE, BLACK);

        sprintf(str, "gyro_x:%.2f", gyro[0]);
        Paint_DrawString_EN(5, 85, str, &Font12, WHITE, BLACK);
        sprintf(str, "gyro_y:%.2f", gyro[1]);
        Paint_DrawString_EN(5, 100, str, &Font12, WHITE, BLACK);
        sprintf(str, "gyro_z:%.2f", gyro[2]);
        Paint_DrawString_EN(5, 115, str, &Font12, WHITE, BLACK);

        OLED_1in5_Display(BlackImage);
        
        DEV_Delay_ms(100);
        Paint_Clear(BLACK);
    }

    return 0;
}
