/*****************************************************************************
* | File      	:   I2C0-EEPROM.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include <stdio.h>
#include "pico/stdlib.h"

#include "OLED_1in5.h"
#include "GUI_Paint.h"
#include "DEV_Config.h"

#include "AT24C02.h"

#define Init_data (0xff)
#define Ploy (0x31)
/******************************************************************************
function: crc8
info:   Read the temperature and humidity sensor values and displayed on the screen
parameter:
    init    :   crc initial value
    poly    :   crc polynomial
    data    :   Data to be validated
    len     :   Data length
******************************************************************************/
uint8_t crc8(uint8_t init, uint8_t poly, uint8_t *data, uint16_t len)
{
    uint8_t crc = init;
    while (len--)
    {
        crc ^= *data++;
        for (uint8_t i = 0; i < 8; i++)
        {
            if (crc & 0x80)
            {
                crc = (crc << 1) ^ poly;
            }
            else
            {
                crc = crc << 1;
            }
        }
    }
    return crc;
}
/******************************************************************************
function: main
info:   Record the current number of restarts
parameter:
******************************************************************************/
int main()
{
    char read_data[128];
    char write_data[128];
    char str[10];
    uint8_t cnt;
    uint8_t crc_data;
    uint16_t reboot_cnt;
    uint8_t *BlackImage;
    uint16_t Imagesize = ((OLED_1in5_WIDTH % 2 == 0) ? (OLED_1in5_WIDTH / 2) : (OLED_1in5_WIDTH / 2 + 1)) * OLED_1in5_HEIGHT;
    
    DEV_Module_Init();
    //Initialize the OLED
    OLED_1in5_Init();
    OLED_1in5_Clear(0);
    if ((BlackImage = (uint8_t *)malloc(Imagesize)) == NULL)
    {
        while (1)
        {
            printf("Failed to apply for black memory...\r\n");
        }
    }
    printf("Paint_NewImage\r\n");
    Paint_NewImage(BlackImage, OLED_1in5_WIDTH, OLED_1in5_HEIGHT, 0, WHITE);
    Paint_SetScale(16);
    Paint_SelectImage(BlackImage);
    Paint_Clear(BLACK);

    Paint_DrawString_EN(5, 10, "EEPROM TEST", &Font12, WHITE, BLACK);
    //READ EEPROM DATA
    AT24C02_Read(0x00, read_data, 3);
    //CRC check
    for (cnt = 0; (read_data[2] != crc8(Init_data, Ploy, read_data, 2)) && (cnt < 10); cnt++)
    {
        static uint8_t cnt = 0;
        write_data[0] = 0;
        write_data[1] = 0;
        write_data[2] = crc8(Init_data, Ploy, write_data, 2);
        AT24C02_Write(0x00, write_data, 3);
        DEV_Delay_ms(10);
        AT24C02_Read(0x00, read_data, 3);
    }
    // CRC FAIL
    if (cnt >= 10)
    {
        printf("EEPROM Test error");
        Paint_DrawString_EN(5, 25, "EEPROM ERROR", &Font12, WHITE, BLACK);
        OLED_1in5_Display(BlackImage);
        return 0;
    }
    reboot_cnt = read_data[0] << 8;
    reboot_cnt += read_data[1];

    if (reboot_cnt >= 1000)
    {
        reboot_cnt = 0;
    }
    sprintf(str, "Reboot cnt = %d", reboot_cnt);
    Paint_DrawString_EN(5, 25, str, &Font12, WHITE, BLACK);
    
    reboot_cnt++;

    write_data[1] = reboot_cnt & 0xff;
    write_data[0] = (reboot_cnt >> 8) & 0xff;
    write_data[2] = crc8(Init_data, Ploy, write_data, 2);
    //write Reboot cnt
    AT24C02_Write(0x00, write_data, 3);

    OLED_1in5_Display(BlackImage);

    return 0;
}
