/*****************************************************************************
* | File      	:   I2C0-SHTC3.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include <stdio.h>
#include "pico/stdlib.h"

#include "OLED_1in5.h"
#include "GUI_Paint.h"
#include "DEV_Config.h"
#include "SHTC3.h"
/******************************************************************************
function: main
info:   Read the temperature and humidity sensor values and displayed on the screen
parameter:
******************************************************************************/
int main()
{
    float temp, hum;
    int16_t id;
    bool crc_res;
    uint8_t *BlackImage;
    uint16_t Imagesize = ((OLED_1in5_WIDTH % 2 == 0) ? (OLED_1in5_WIDTH / 2) : (OLED_1in5_WIDTH / 2 + 1)) * OLED_1in5_HEIGHT;

    if (DEV_Module_Init() != 0)
    {
        return -1;
    }
    //Initialize the OLED
    OLED_1in5_Init();
    OLED_1in5_Clear(0);
    // Initialize the temperature and humidity sensor
    SHTC3_Init();
    if ((BlackImage = (uint8_t *)malloc(Imagesize)) == NULL)
    {
        while (1)
        {
            printf("Failed to apply for black memory...\r\n");
        }
    }
    printf("Paint_NewImage\r\n");
    Paint_NewImage(BlackImage, OLED_1in5_WIDTH, OLED_1in5_HEIGHT, 0, WHITE);
    Paint_SetScale(16);
    Paint_SelectImage(BlackImage);

    while (1)
    {
        char str[20];
        Paint_Clear(BLACK);
        crc_res = SHTC3_Measurement(&temp, &hum);  
        id = SHTC3_Read_Id();
        // Read and validate the data
        if (id == -1 || crc_res == false)
        {
            Paint_DrawString_EN(5, 40, "CRC8 fail", &Font12, WHITE, BLACK);
            OLED_1in5_Display(BlackImage);
            printf("Error： Measurement CRC8 check failure \r\n");
            break;
        }
        Paint_DrawString_EN(5, 10, "T&H TEST", &Font12, WHITE, BLACK);
        Paint_DrawString_EN(5, 25, "IC:SHTC3", &Font12, WHITE, BLACK);

        sprintf(str, "ID : 0x%x", id);
        Paint_DrawString_EN(5, 40, str, &Font12, WHITE, BLACK);

        sprintf(str, "TEMP : %.2f", temp);
        Paint_DrawString_EN(5, 55, str, &Font12, WHITE, BLACK);
        sprintf(str, "RH   : %.2f", hum);
        Paint_DrawString_EN(5, 70, str, &Font12, WHITE, BLACK);

        printf("T= %.2f , RH = %.2f  \r\n", temp, hum);
        OLED_1in5_Display(BlackImage);
        DEV_Delay_ms(1000);
    }

    return 0;
}
