/*****************************************************************************
* | File      	:   ADC2-SOUND.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include "OLED_1in5.h"
#include "GUI_Paint.h"
#include "DEV_Config.h"

#define Sound_Dout_Pin (21)
#define Sound_Aout_Pin (28)
/******************************************************************************
function: ADC_pin_init
info:   Set GPIO to adc pin
parameter:
    gpio    :   Set gpio to key
******************************************************************************/
void ADC_pin_init(uint gpio)
{
    adc_init();
    if (gpio < 26 || gpio > 29)
    {
        return;
    }
    // Make sure GPIO is high-impedance, no pullups etc
    adc_gpio_init(gpio);
    // Select ADC input 0 (GPIO26)
    adc_select_input(gpio - 26);
}
/******************************************************************************
function: ADC_read_vol
info:   read adc value , converted to a voltage value and return
parameter:
******************************************************************************/
float ADC_read_vol()
{
    const float conversion_factor = 3.3f / (1 << 12);
    uint16_t result = adc_read();
    return result * conversion_factor;
}
/******************************************************************************
function: key_init
info:   Set GPIO to input 
parameter:
    gpio    :   Set gpio to key
******************************************************************************/
void key_init(uint gpio)
{
    gpio_init(gpio);
    gpio_set_dir(gpio, GPIO_IN);
}
/******************************************************************************
function: main
info:   Output current sound sensor Aout voltage with Dout value
parameter:
******************************************************************************/
int main()
{
    uint8_t *BlackImage;
    uint16_t Imagesize = ((OLED_1in5_WIDTH % 2 == 0) ? (OLED_1in5_WIDTH / 2) : (OLED_1in5_WIDTH / 2 + 1)) * OLED_1in5_HEIGHT;

    if (DEV_Module_Init() != 0)
    {
        return -1;
    }
    //Initialize the OLED
    OLED_1in5_Init();
    OLED_1in5_Clear(0);
    if ((BlackImage = (uint8_t *)malloc(Imagesize)) == NULL)
    {
        while (1)
        {
            printf("Failed to apply for black memory...\r\n");
        }
    }
    printf("Paint_NewImage\r\n");
    Paint_NewImage(BlackImage, OLED_1in5_WIDTH, OLED_1in5_HEIGHT, 0, WHITE);
    Paint_SetScale(16);
    Paint_SelectImage(BlackImage);

    ADC_pin_init(Sound_Aout_Pin);
    key_init(Sound_Dout_Pin);
    printf("Hello, SOUND!");
    while (true)
    {
        bool dout;
        float vol;
        char str[20];
        Paint_Clear(BLACK);

        dout=gpio_get(Sound_Dout_Pin);
        vol=ADC_read_vol();

        Paint_DrawString_EN(5, 10, "SOUND TEST", &Font12, WHITE, BLACK);

        sprintf(str, "Aout_Pin = %.2fV", vol);
        Paint_DrawString_EN(5, 25, str, &Font12, WHITE, BLACK);

        sprintf(str, "Dout_Pin = %d", dout);
        Paint_DrawString_EN(5, 40, str, &Font12, WHITE, BLACK);

        OLED_1in5_Display(BlackImage);

        printf("Sound_Aout_Pin Voltage = %.2fV \r\n",vol);
        printf("Sound_Dout_Pin = %d \r\n",dout);
        sleep_ms(100);
    }


    return 0;
}
