/*****************************************************************************
* | File      	:   D3-IR-RX.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include "OLED_1in5.h"
#include "GUI_Paint.h"
#include "DEV_Config.h"
#include "nec_receive.h"

#define PIO_NUM (pio0)
#define IR_RX_PIN (15)

/******************************************************************************
function: main
info:   After receiving the infrared remote control signal, 
        decode and output the data through the serial port
parameter:
******************************************************************************/
int main()
{
    uint8_t rx_address, rx_data;
    PIO pio = PIO_NUM; // choose which PIO block to use (RP2040 has two: pio0 and pio1)
    uint8_t *BlackImage;
    uint16_t Imagesize = ((OLED_1in5_WIDTH % 2 == 0) ? (OLED_1in5_WIDTH / 2) : (OLED_1in5_WIDTH / 2 + 1)) * OLED_1in5_HEIGHT;

    if (DEV_Module_Init() != 0)
    {
        return -1;
    }
    //Initialize the OLED
    OLED_1in5_Init();
    OLED_1in5_Clear(0);
    if ((BlackImage = (uint8_t *)malloc(Imagesize)) == NULL)
    {
        while (1)
        {
            printf("Failed to apply for black memory...\r\n");
        }
    }
    printf("Paint_NewImage\r\n");
    Paint_NewImage(BlackImage, OLED_1in5_WIDTH, OLED_1in5_HEIGHT, 0, WHITE);
    Paint_SetScale(16);
    Paint_SelectImage(BlackImage);

    int rx_sm = nec_rx_init(pio, IR_RX_PIN); // uses one state machine and 9 instructions
    if (rx_sm == -1)
    {
        printf("could not configure PIO\n");
        return -1;
    }
    printf("Hello,IR RX \r\n");
    char str[20];
    Paint_Clear(BLACK);
    Paint_DrawString_EN(5, 10, "Wait receiving", &Font12, WHITE, BLACK);
    OLED_1in5_Display(BlackImage);
    while (true)
    {
        //wait receiving the infrared remote control signal
        while (!pio_sm_is_rx_fifo_empty(pio, rx_sm))
        {
            uint32_t rx_frame = pio_sm_get(pio, rx_sm);
            Paint_Clear(BLACK);
            Paint_DrawString_EN(5, 10, "IR RX TEST", &Font12, WHITE, BLACK);

            if (nec_decode_frame(rx_frame, &rx_address, &rx_data))
            {
                sprintf(str, "received: %02x, %02x ", rx_address, rx_data);
                Paint_DrawString_EN(5, 25, str, &Font12, WHITE, BLACK);
                printf("received: %02x, %02x \r\n", rx_address, rx_data);
            }
            else
            {
                sprintf(str, "received: %08x ", rx_frame);
                Paint_DrawString_EN(5, 25, str, &Font12, WHITE, BLACK);
                printf("received: %08x \r\n", rx_frame);
            }
            OLED_1in5_Display(BlackImage);
        }
        sleep_ms(10);
    }

    return 0;
}
