/*****************************************************************************
* | File      	:   D4-IR-TX.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include "OLED_1in5.h"
#include "GUI_Paint.h"
#include "DEV_Config.h"
#include "nec_transmit.h"  

#define PIO_NUM (pio0)
#define IR_TX_PIN (16)

/******************************************************************************
function: main
info:   Every second by emitting an infrared signal
parameter:
******************************************************************************/
int main()
{
     uint8_t *BlackImage;
    uint16_t Imagesize = ((OLED_1in5_WIDTH % 2 == 0) ? (OLED_1in5_WIDTH / 2) : (OLED_1in5_WIDTH / 2 + 1)) * OLED_1in5_HEIGHT;

    if (DEV_Module_Init() != 0)
    {
        return -1;
    }
    //Initialize the OLED
    OLED_1in5_Init();
    OLED_1in5_Clear(0);
    if ((BlackImage = (uint8_t *)malloc(Imagesize)) == NULL)
    {
        while (1)
        {
            printf("Failed to apply for black memory...\r\n");
        }
    }
    printf("Paint_NewImage\r\n");
    Paint_NewImage(BlackImage, OLED_1in5_WIDTH, OLED_1in5_HEIGHT, 0, WHITE);
    Paint_SetScale(16);
    Paint_SelectImage(BlackImage);

    PIO pio = PIO_NUM; 
    uint8_t tx_address = 0x00, tx_data = 0x00; 
    // ir tx init
    int tx_sm = nec_tx_init(pio, IR_TX_PIN); 
    char str[20];
    printf("Hello,IR TX \r\n");
    while (true) {
        //get encode
        uint32_t tx_frame = nec_encode_frame(tx_address, tx_data);
        //emitting an infrared signal
        pio_sm_put(pio, tx_sm, tx_frame);
        Paint_Clear(BLACK);
        Paint_DrawString_EN(5, 10, "IR TX TEST", &Font12, WHITE, BLACK);
        sprintf(str, "sent: %02x, %02x ", tx_address, tx_data);
        Paint_DrawString_EN(5, 25, str, &Font12, WHITE, BLACK);
        printf("sent: %02x, %02x \r\n", tx_address, tx_data);
        OLED_1in5_Display(BlackImage);
        sleep_ms(1000);
        tx_data += 1;
    }


    return 0;
}
