/*****************************************************************************
* | File      	:   D7-RGB-LED.c
* | Author      :   Waveshare Team
* | Function    :
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2023-02-01
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
******************************************************************************/
#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/pio.h"
#include "ws2812.pio.h"

#define PIO_NUM (pio0)
#define WS2812_PIN (22)
/******************************************************************************
function: put_pixel
info:   Send rgb data
parameter:
    pixel_grb   :   rgb data
******************************************************************************/
void put_pixel(uint32_t pixel_grb)
{
    pio_sm_put_blocking(PIO_NUM, 0, pixel_grb << 8u);
}
/******************************************************************************
function: put_rgb
info:   Color breathing lamp
parameter:
    red         :   red data
    green       :   green data
    blue        :   blue data   
******************************************************************************/
void put_rgb(uint8_t red, uint8_t green, uint8_t blue)
{
    uint32_t mask = (green << 16) | (red << 8) | (blue << 0);
    put_pixel(mask);
}
/******************************************************************************
function: main
info:   Color breathing lamp
parameter:
******************************************************************************/
int main()
{

    int sm = 0;
    uint8_t cnt = 0;
    uint offset = pio_add_program(PIO_NUM, &ws2812_program);

    stdio_init_all();
    // RGB LED PIO init
    ws2812_program_init(PIO_NUM, sm, offset, WS2812_PIN, 800000, true);

    int t = 0;
    while (1)
    {
        for (cnt = 0; cnt < 0xff; cnt++)
        {
            put_rgb(cnt, 0xff - cnt, 0);
            sleep_ms(3);
        }
        for (cnt = 0; cnt < 0xff; cnt++)
        {
            put_rgb(0xff - cnt, 0, cnt);
            sleep_ms(3);
        }
        for (cnt = 0; cnt < 0xff; cnt++)
        {
            put_rgb(0, cnt, 0xff - cnt);
            sleep_ms(3);
        }
    }

}
